<?php
/**
 * @package CCL
 * @version 0.0.1
 * @author trustmaster
 * @copyright (c) 2009 Vladimir Sibirov
 * @license BSD
 */

/**
 * Abstract class with properties contained as associative array and simple
 * methods to access them.
 */
abstract class Container implements ArrayAccess, Iterator, Serializable {
    /**
	 * Fields containing values
	 * @var array Contained data
	 */
	protected $data = array();

	/**
	 * Constructs container and puts data inside
	 * @param array $data Property set
	 */
	public function __construct($data)
	{
		if(is_array($data))
		{
			$this->data = $data;
		}
	}

	/**
	 * Returns current container element
	 * @return mixed Data element
	 */
	public function current()
	{
		return current($this->data);
	}

	/**
	 * Fetches current key from the container
	 * @return mixed Current key
	 */
	public function key()
	{
		return key($this->data);
	}

	/**
	 * Checks if key exists in container
	 * @param mixed $key Key to search for
	 * @return bool TRUE if key exists, FALSE otherwise
	 */
	public function key_exists($key)
	{
		return key_exists($key, $this->data);
	}

	/**
	 * Fetches next element from the container
	 * @return mixed Next element
	 */
	public function next()
	{
		return next($this->data);
	}

	/**
	 * Sets a contained item value in array style
	 * @param mixed $offset Key
	 * @param mixed $value  Value
	 */
	public function offsetSet($offset, $value)
	{
		$this->data[$offset] = $value;
	}

	/**
	 * Checks if any item is assigned to a key
	 * @param mixed $offset Key
	 * @return bool Item exists
	 */
	public function offsetExists($offset)
	{
		return isset($this->data[$offset]);
	}

	/**
	 * Removes contained item by key
	 * @param mixed $offset Key
	 */
	public function offsetUnset($offset)
	{
		unset($this->data[$offset]);
	}

	/**
	 * Returns contained item by key or NULL on error
	 * @param mixed $offset Key
	 * @return mixed Contained item or NULL if none exists
	 */
	public function offsetGet($offset)
	{
		return isset($this->data[$offset]) ? $this->data[$offset] : NULL;
	}

	/**
	 * Sets iterator pointer to the beginning of the container
	 */
	public function rewind()
	{
		reset($this->data);
	}

	/**
	 * Serializes the container
	 * @return string Serialized data
	 */
	public function serialize()
	{
		return serialize($this->data);
	}

	/**
	 * Checks if iterator pointer points to a valid element (non-false)
	 * @return bool Current element validity
	 */
	public function valid()
	{
		return $this->current() !== FALSE;
	}

	/**
	 * Unserializes container data
	 * @param string $srl_data Serialized data
	 */
	public function unserialize($srl_data)
	{
		$this->data = unserialize($srl_data);
	}
}
?>
