<?php
/**
 * @package CCL
 * @version 0.0.1
 * @author trustmaster
 * @copyright (c) 2009 Vladimir Sibirov
 * @license BSD
 */

/**
 * Date parameter class. In Cotonti all date/time types are UNIX timestamps inside
 */
class DateStamp extends Param
{
	/**
	 * Entity parameter constructor
	 * 
	 * @param string $name Parameter name
	 * @param bool $display Render it in templates
	 * @param bool $required Require it from user input
	 * @param bool $primary_key Is primary key of the entity
	 */
	public function __construct($name, $display = TRUE, $required = FALSE, $primary_key = FALSE)
	{
		parent::__construct(Param_type::int, $name, $display, $required, $primary_key);
	}

	/**
	 * Imports parameter value from user input
	 * @param string $method Request method: P - post, G - get.
	 * @param string $prefix Input name prefix
	 * @return int Value on success or NULL if validation failed
	 */
	public function import($method = 'P', $prefix = '')
	{
		// TODO check $this->required, throw exceptions
		// First try importing as string, if it fails import by parts
		$date = sed_import($prefix . $this->name, $method, 'STX');
		if (preg_match('#(\d{4})-(\d{2})-(\d{2})#', $date, $mt))
		{
			if (!empty($mt[1]))
			{
				$stamp = mktime(0, 0, 0, (int) $mt[2], (int) $mt[3], (int) $mt[1]);
			}
			else
			{
				$stamp = NULL;
			}
		}
		else
		{
			$year = (int) sed_import('ryear_' . $prefix . $this->name, $method, 'INT');
			$month = (int) sed_import('rmonth_' . $prefix . $this->name, $method, 'INT');
			$day = (int) sed_import('rday_' . $prefix . $this->name, $method, 'INT');
			if ($year == 0)
			{
				$stamp = NULL;
			}
			else
			{
				$stamp = mktime(0, 0, 0, $month, $day, $year);
			}
		}
		if (empty($stamp) && $this->required)
		{
			$stamp = NULL;
		}
		elseif (empty($stamp) && !$this->required && !is_null($this->default))
		{
			$stamp = $this->default;
		}
		return $stamp;
	}

	/**
	 * Renders param value display in a correct way
	 * @param mixed $value
	 * @return string Rendered display element
	 * @see Date::display_input(), Param::title, Param::hint
	 */
	public function display($value)
	{
		global $cfg;
		$disp = date($cfg['formatyearmonthday'], $value);
		return $disp;
	}

	/**
	 * Renders param input display in a correct way
	 * @param mixed $value
	 * @param string $prefix Input name prefix
	 * @param bool $separate Display date parts separately
	 * @return string
	 * @see Date::display(), Param::title, Param::hint
	 */
	public function display_input($value = NULL, $prefix = '', $separate = TRUE)
	{
		global $sys;
		if (is_null($value) || empty($value))
		{
			if (!is_null($this->default))
			{
				$value = $this->default;
			}
			elseif ($this->required)
			{
				$value = $sys['now_offset'];
			}
		}
		// FIXME $class = empty($this->input_class) ? '' : ' class="' . $this->input_class . '"';
		$disp = sed_selectbox_date($value, 'short', '_' . $prefix . $this->name);
		return $disp;
	}
}
?>
