<?php
/**
 * @package CCL
 * @version 0.0.1
 * @author trustmaster
 * @copyright (c) 2009 Vladimir Sibirov
 * @license BSD
 */

/**
 * Data model entity in terms of ER design.
 */
class Entity extends Container {
	/**
	 * @var mixed Name of the primary key parameter or array of names for compound key
	 */
	public $primary_key = 'id';
	/**
	 * Global entity name
	 * @var string Entity name
	 */
	public $name;
	/**
	 * @var string SQL table name prefix (e.g. 'sed_')
	 */
	public $table_prefix = '';
	/**
	 * @var string Full SQL table name 
	 */
	public $table_name = '';
	/**
	 * @var string SQL column name prefix (e.g. 'ent_')
	 */
	public $col_prefix = '';

	/**
	 * Entity constructor
	 * @param string $name Entity name
	 * @param string $table_prefix Table prefix, e.g. 'cot_'
	 * @param string $col_prefix SQL column prefix
	 * @param array $params Table parameters
	 * @param string $primary_key Primary key field(s)
	 */
	public function __construct($name, $table_prefix = '', $col_prefix = '', $params = NULL, $primary_key = 'id')
	{
		$this->name = $name;
		$this->col_prefix = $col_prefix;
		$this->table_prefix = $table_prefix;
		$this->table_name = $table_prefix . $name;
		if (is_array($params))
		{
			$this->define($params, $primary_key);
		}
	}

	/**
	 * Adds a new entry to the entity parameter list. After the parameter has
	 * been added, you will be able to access it by name in associative array
	 * style.
	 *
	 * @param Param $param Parameter object to be added (can be Param, Ref or Dict)
	 * @return Param Reference to the added parameter
	 * @see Param, Reference, Dict
	 */
	public function add($param)
	{
		$this->data[$param->name] = $param;
		if($param->primary_key)
		{
			$this->primary_key = $param->name;
		}
		return $this->data[$param->name];
	}

	/**
	 * Defines entity structure all at once
	 *
	 * @param array $params Array of Param
	 * @param mixed $primary_key Primary key name(s)
	 * @see Entity::add()
	 */
	public function define($params, $primary_key = 'id')
	{
		if (is_array($params))
		{
			$this->data = $params;
			$this->primary_key = $primary_key;
		}
	}

	/**
	 * Fetches Param used as primary key for the entity
	 * @return Param Primary key parameter
	 */
	public function get_primary_key()
	{
		return $this->data[$this->primary_key];
	}
}
?>
