<?php
/**
 * @package CCL
 * @version 0.0.1
 * @author trustmaster
 * @copyright (c) 2009 Vladimir Sibirov
 * @license BSD
 */

/**
 * Instance of entity as data entry
 */
class Instance {
	/**
	 * @var Entity Reference to an entity this object instantiates
	 */
    protected $entity;
	/**
	 * @var array Data fields
	 */
	protected $data = array();

	/**
	 * Creates object from SQL result row or array of parameter values or
	 * blank to be filled later.
	 *
	 * @param Entity $entity Reference to parent entity
	 * @param mixed $source ID or query result row
	 */
	public function __construct($entity, $source = NULL)
	{
		$this->entity = $entity;
		if (is_array($source))
		{
			// Got a row from db, assign data
			$this->assign_from_row($source);
		}
	}

	/**
	 * Checks whether instance exists in database
	 * @return bool
	 */
	public function exists()
	{
		$key = $this->get_key();
		if (!$key) return FALSE;
		$res = sed_sql_query("SELECT COUNT(*) FROM `" . $this->entity->table_name . "` WHERE " . $key);
		return sed_sql_result($res, 0, 0) == 1;
	}

	/**
	 * @return mixed SQL primary key for WHERE
	 */
	protected function get_key()
	{
		if (empty($this->id) || !is_int($this->id)) return FALSE;
		return '`' . $this->entity->col_prefix . 'id` = ' . $this->id;
	}

	/**
	 * Fills object data using SQL result row
	 * @param array $row SQL query result row
	 * @return bool TRUE on success, FALSE on error
	 */
	private function assign_from_row($row)
	{
		if (is_array($row))
		{
			foreach ($row as $key => $val)
			{
				if (preg_match('#^' . $this->entity->col_prefix . '([\w_]+)$#', $key, $mt))
				{
					$name = $mt[1];
				}
				else
				{
					$name = $key;
				}
				if ($this->entity->key_exists($name))
				{
					$this->{$name} = $val; // __set()
				}
			}
			return TRUE;
		}
		else
		{
			return FALSE;
		}
	}

	/**
	 * Returns property value
	 * @param string $name Property name
	 * @return mixed Property value
	 */
	public function __get($name)
	{
		return $this->data[$name];
	}

	/**
	 * Checks for property
	 * @param string $name Property name
	 * @return bool Whether field is set
	 */
	public function __isset($name)
	{
		return isset($this->data[$name]);
	}

	/**
	 * Sets property value
	 * @param string $name Property name
	 * @param mixed $value Property value
	 */
	public function __set($name, $value)
	{
		if (isset($this->entity[$name]))
			$this->data[$name] = $this->entity[$name]->cast($value);
		else
			$this->data[$name] = $value;
	}

	/**
	 * Unsets property value
	 * @param string $name Property name
	 */
	public function __unset($name)
	{
		unset($this->data[$name]);
	}

	/**
	 * Removes object instance from the database
	 * @return bool TRUE on success, FALSE on error
	 */
	public function delete()
	{
		$id = $this->get_key();
		if (!$id) return FALSE;

		$res = sed_sql_query('DELETE FROM `' . $this->entity->table_name . '`
			WHERE ' . $id );

		return sed_sql_affectedrows() == 1;
		// TODO implement automatic removal of referenced objects
	}

	/**
	 * Saves unit object in database. Performs UPDATE if object ID is set or INSERT otherwise.
	 * @return mixed FALSE on error, TRUE on successful update or ID on successful insert
	 */
	public function save()
	{
		$sql_keys = array();
		$sql_vals = array();
		foreach ($this->entity as $name => $param)
		{
			if (!is_null($this->{$name}))
			{
				$sql_keys[] = $this->entity->col_prefix . $name;
				$sql_vals[] = $param->prepare($this->{$name});
			}
		}

		$count = count($sql_keys);
		if ($count == 0)
		{
			return FALSE;
		}

		$id = $this->get_key();
		$exists = $this->exists();

		if (!$exists)
		{
			// Insert new item
			$query = 'INSERT INTO `' . $this->entity->table_name . '` (';
			foreach ($sql_keys as $key)
			{
				$query .= "`$key`,";
			}
			$query = mb_substr($query, 0, -1);
			$query .= ') VALUES (';
			foreach ($sql_vals as $val)
			{
				$query .= "$val,";
			}
			$query = mb_substr($query, 0, -1);
			$query .= ')';
			$res = sed_sql_query($query);
			return sed_sql_insertid();
		}
		else
		{
			// Update existing item
			$query = 'UPDATE `' . $this->entity->table_name . '` SET ';
			for ($i = 0; $i < $count; $i++)
			{
				$query .= "`{$sql_keys[$i]}` = {$sql_vals[$i]},";
			}
			$query = mb_substr($query, 0, -1);
			$query .= ' WHERE ' . $id;

			$res = sed_sql_query($query);
			return sed_sql_affectedrows() == 1;
		}
	}
}
?>
