<?php
/**
 * @package CCL
 * @version 0.0.1
 * @author trustmaster
 * @copyright (c) 2009 Vladimir Sibirov
 * @license BSD
 */

/**
 * A Link is an Item that belongs to Relation and links Items from several Units together
 */
class Link extends Item {
	/**
	 * @var string Displayed identifier
	 */
	public $name = '';
	/**
	 * Instance constructor. Creates object from SQL result row, loads by ID or just leaves
	 * blank to be filled later.
	 * @param Unit $unit Reference to parent unit
	 * @param array $source Query result row or array with params
	 */
	public function __construct($unit, $source = NULL)
	{
		parent::__construct($unit, $source);
		// Combine identifier
		foreach ($this->data as $key => $val)
		{
			$this->name .= $key.$val;
		}
	}

	/**
	 * Row-based automatic unit render
	 * TPL designer specifies parameters implicitly with row block.
	 * TPL example:
	 * <!-- BEGIN: MY_UNIT -->
	 * <!-- BEGIN: MY_PARAM -->
	 * <stong>{PARAM_TITLE}:</strong> {PARAM_VALUE}
	 * <!-- END: MY_PARAM -->
	 * <!-- END: MY_UNIT -->
	 *
	 * @param XTemplate $tpl Layout template object
	 * @param string $block Full block name to render the object in, e.g. 'MAIN.MY_UNIT'
	 * @param string $row_block Row block name for parameter row, e.g. 'MY_PARAM'
	 * @param string $title_tag Tag name for parameter title
	 * @param string $value_tag Tag name for parameter value
	 */
	public function display($tpl, $block, $row_block = 'PARAM', $title_tag = 'PARAM_TITLE', $value_tag = 'PARAM_VALUE')
	{
		// TODO check edit permissions
		$id = '';
		foreach ($this->data as $name => $val)
		{
			$id .= "&$name=$val";
		}
		if (defined('SED_MODULE'))
		{
			$tpl->assign(array(
				'DISPLAY_URL' => sed_url(SED_MODULE, 'z=' . $this->entity->zone . '&m=display' . $id),
				'EDIT_URL' => sed_url(SED_MODULE, 'z=' . $this->entity->zone . '&m=edit' . $id),
				'REMOVE_URL' => sed_url(SED_MODULE, 'z=' . $this->entity->zone . '&m=remove' . $id)
			));
		}
		else
		{
			global $pl;
			$tpl->assign(array(
				'DISPLAY_URL' => sed_url('plug', 'e=' . $pl['pl_code'].'&z=' . $this->entity->zone . '&m=display' . $id),
				'EDIT_URL' => sed_url('plug', 'e=' . $pl['pl_code'].'&z=' . $this->entity->zone . '&m=edit' . $id),
				'REMOVE_URL' => sed_url('plug', 'e=' . $pl['pl_code'].'&z=' . $this->entity->zone . '&m=remove' . $id)
			));
		}
		parent::display_auto($tpl, $block, $row_block, $title_tag, $value_tag);
	}

	/**
	 * Checks whether instance exists in database
	 * @return bool
	 */
	public function exists()
	{
		$key = $this->get_key();
		if (!$key) return FALSE;
		$res = sed_sql_query("SELECT COUNT(*) FROM `" . $this->entity->table_name . "` WHERE " . $key);
		return sed_sql_result($res, 0, 0) == 1;
	}

	/**
	 * @return mixed Primary key value for SQL WHERE
	 */
	protected function get_key()
	{
		global $id;
		$pri_key = array();
		foreach ($this->data as $key => $val)
		{
			if (is_array($id) && !empty($id[$key])) 
			{
				$val = $id[$key];
			}
			$pri_key[] = "`{$this->entity->col_prefix}$key` = " . $this->entity[$key]->prepare($val);
		}
		if (count($pri_key) == 0) return FALSE;
		$pri_key = implode(' AND ', $pri_key);
		return $pri_key;
	}
}
?>
