<?php
/**
 * @package CCL
 * @version 0.0.1
 * @author trustmaster
 * @copyright (c) 2009 Vladimir Sibirov
 * @license BSD
 */

/**
 * A reference parameter in terms of relational database design
 */
class Reference extends Param {
	/**
	 * @var Entity Referenced entity
	 */
	public $entity;
	/**
	 * If caption parameter is not specified, the key will be used itself
	 * @var string Referenced caption field name displayed in the dict list
	 */
	public $caption = '';
	/**
	 * Is used to cache referenced keys/values not to query them every time
	 * @var array Key/val cache
	 */
	public $cache = array();

	/**
	 *
	 * @param string $name Parameter name
	 * @param Entity $entity Referenced entity
	 * @param string $caption Parameter of referenced entity displayed as a caption
	 * @param bool $display Render it in templates
	 * @param bool $required Require non-empty value from user input
	 */
	public function __construct($name, $entity, $caption = '', $display = TRUE, $required = FALSE)
	{
		if(empty($entity))
		{
			throw new Exception('Dictionary parameter references empty entity');
		}
		$this->entity = $entity;
		$this->caption = $caption;

		parent::__construct(Param_type::int, $name, $display, $required);

		$this->refresh();
		// TODO global cache for references
	}

	/**
	 * Refreshes key/value cache for the reference
	 */
	public function refresh()
	{
		$key = $this->entity->col_prefix . $this->entity->primary_key;
		$caption = $this->entity->col_prefix . $this->caption;
		$this->cache = array();
		$res = sed_sql_query("SELECT `$key`, `$caption` FROM `{$this->entity->table_name}`");
		while ($row = sed_sql_fetchassoc($res))
		{
			$this->cache[$row[$key]] = $row[$caption];
		}
		sed_sql_freeresult($res);
	}
	
	/**
	 * Renders param value display in a correct way
	 * @param mixed $value
	 * @return string Rendered display element
	 * @see Param::display_input(), Param::title, Param::hint
	 */
	public function display($value)
	{
		$disp = $this->cache[$value];
		return $disp;
	}

	/**
	 * Renders param input display in a correct way
	 * @param mixed $value
	 * @param string $prefix Input name prefix
	 * @param int $exclude Exclude this item from list (usually self)
	 * @return string
	 * @see Param::title, Param::hint
	 */
	public function display_input($value = NULL, $prefix = '', $exclude = NULL)
	{
		if (is_null($value))
		{
			$filter = sed_import($this->name, 'G', 'INT');
			if (!empty($filter))
			{
				$value = $filter;
			}
			elseif (!is_null($this->default))
			{
				$value = $this->default;
			}
		}
		$class = empty($this->input_class) ? '' : ' class="' . $this->input_class . '"';
		$disp = '<select name="' . $prefix . $this->name . '"' . $class . '>';
		if (!$this->required)
		{
			$disp .= '<option value="">--</option>';
		}
		foreach ($this->cache as $key => $capt)
		{
			if ($key != $exclude)
			{
				$selected = (is_null($value) || $value != $key) ? '' : ' selected="selected"';
				$disp .= '<option value="' . sed_cc($key) . '"' . $selected . '>' . sed_cc($capt) . '</option>';
			}
		}
		$disp .= '</select>';
		return $disp;
	}

	/**
	 * Gets a result row for referenced item which you can pass to constructor of exact class
	 * @param int $value Reference key value
	 * @return array SQL result row
	 */
	public function get_item($value)
	{
		$value = (int) $value;
		$res = sed_sql_query("SELECT * FROM `{$this->entity->table_name}`
			WHERE `{$this->entity->primary_key}` = $value");
		$cnt = sed_sql_numrows($res);
		if ($cnt == 0)
		{
			return NULL;
		}
		else
		{
			return sed_sql_fetchassoc($res);
		}
	}
}
?>
