<?php
/**
 * @package CCL
 * @version 0.0.1
 * @author trustmaster
 * @copyright (c) 2009 Vladimir Sibirov
 * @license BSD
 */

/**
 * Set parameter class
 */
class Set extends Enum
{
	/**
	 * Imports parameter value from user input
	 * @param string $method Request method: P - post, G - get.
	 * @param string $prefix Input name prefix
	 * @return array Value on success or NULL if validation failed
	 */
	public function import($method = 'P', $prefix = '')
	{
		// TODO throw exceptions
		$values = $_POST[$prefix . $this->name];
		$data = array();
		if (is_array($values) && count($values) > 0)
		{
			foreach ($values as $value)
			{
				if (in_array($value, $this->values))
				{
					$data[] = $value;
				}
			}
		}

		if (count($data) == 0)
		{
			if (!$this->required && !is_null($this->default))
			{
				return $this->default;
			}
			elseif (!$this->required)
			{
				return '';
			}
			else
			{
				return NULL;
			}
		}
		return $data;
	}

	/**
	 * Prepares a parameter value for use in database statement
	 * @param array $value Source value
	 * @return string Prepared value
	 */
	public function prepare($value)
	{
		if (is_array($value))
		{
			return "'" . sed_sql_prep(implode(',', $value)) . "'";
		}
		else
		{
			return "'" . sed_sql_prep($value) . "'";
		}
	}

	/**
	 * Renders param value display in a correct way
	 * @param array $value
	 * @return string Rendered display element
	 */
	public function display($value)
	{
		if (is_string($value)) $value = explode(',', $value);
		if (is_array($value))
		{
			foreach ($value as $val)
			{
				$disp .= parent::display($val) . ', ';
			}
			if (!empty($disp)) $disp = mb_substr($disp, 0, -2);
		}
		return $disp;
	}

	/**
	 * Renders param input display in a correct way
	 * @param mixed $value
	 * @param string $prefix Input name prefix
	 * @return string
	 * @see Param::title, Param::hint
	 */
	public function display_input($value = NULL, $prefix = '')
	{
		if (is_null($value) && !is_null($this->default))
		{
			$value = $this->default;
		}
		if (is_string($value)) $value = explode(',', $value);
		$class = empty($this->input_class) ? '' : ' class="' . $this->input_class . '"';
		$disp = '<select name="' . $prefix . $this->name . '[]"' . $class . ' multiple="multiple">';
		$cnt = count($this->values);
		for($i = 0; $i < $cnt; $i++)
		{
			$selected = (is_null($value) || !in_array($this->values[$i], $value)) ? '' : ' selected="selected"';
			$disp .= '<option value="' . sed_cc($this->values[$i]) . '"' . $selected . '>'
				. sed_cc($this->captions[$i]) . '</option>';
		}
		$disp .= '</select>';
		return $disp;
	}
}
?>
